package com.jianshui.invoice.service;

import java.util.List;
import com.jianshui.invoice.domain.Redinfo;

/**
 * 红字信息表Service接口
 * 
 * @author Gong Quanlin
 * @date 2022-03-29
 */
public interface IRedinfoService 
{
    /**
     * 查询红字信息表
     * 
     * @param id 红字信息表主键
     * @return 红字信息表
     */
    public Redinfo selectRedinfoById(Long id);

    /**
     * 查询红字信息表列表
     * 
     * @param redinfo 红字信息表
     * @return 红字信息表集合
     */
    public List<Redinfo> selectRedinfoList(Redinfo redinfo);

    /**
     * 新增红字信息表
     * 
     * @param redinfo 红字信息表
     * @return 结果
     */
    public int insertRedinfo(Redinfo redinfo);

    /**
     * 修改红字信息表
     * 
     * @param redinfo 红字信息表
     * @return 结果
     */
    public int updateRedinfo(Redinfo redinfo);

    /**
     * 批量删除红字信息表
     * 
     * @param ids 需要删除的红字信息表主键集合
     * @return 结果
     */
    public int deleteRedinfoByIds(Long[] ids);

    /**
     * 删除红字信息表信息
     * 
     * @param id 红字信息表主键
     * @return 结果
     */
    public int deleteRedinfoById(Long id);
}
